@@ -20,60 +20,68 @@
 #include <QHBoxLayout>
 #include <QWidget>
 #include <QPainter>
+#include <QtConcurrent/QtConcurrentRun>
 #include <QFile>
 #include <QKeyEvent>
 #include <QSettings>
 #include <QDir>
 #include <QDebug>
 #include <QTimer>
-#include "common/configuration.h"
 #include "xeventmonitor.h"
-
+#include "common/configuration.h"
 
 IconEdit::IconEdit(QWidget *parent)
     : QWidget(parent),
-	  m_configuration(Configuration::instance()),
-      m_timer(nullptr)
+      m_timer(nullptr),
+      m_configuration(Configuration::instance())
 {
     m_edit = new QLineEdit(this);
     m_edit->setObjectName(QStringLiteral("passwdEdit"));
-    m_edit->setAttribute(Qt::WA_InputMethodEnabled, false); //禁用输入法
-    m_edit->setContextMenuPolicy(Qt::NoContextMenu);    //禁用右键菜单
-    m_edit->installEventFilter(this);
 
     m_capsIcon = new QLabel(this);
     m_capsIcon->setObjectName(QStringLiteral("capsIconLabel"));
 
     m_iconButton = new QPushButton(this);
     m_iconButton->setObjectName(QStringLiteral("loginButton"));
-    m_iconButton->setFocusPolicy(Qt::NoFocus);
-    m_iconButton->setCursor(QCursor(Qt::PointingHandCursor));
-    m_iconButton->installEventFilter(this);
+
 
     m_modeButton = new QPushButton(this);
     m_modeButton->setObjectName(QStringLiteral("echoModeButton"));
-    m_modeButton->setCheckable(true);
-    m_modeButton->setFocusPolicy(Qt::NoFocus);
-    m_modeButton->setCursor(Qt::PointingHandCursor);
-    connect(m_modeButton, &QPushButton::clicked, this, [&](bool checked){
-        setType(checked ? QLineEdit::Normal : QLineEdit::Password);
-    });
 
     QHBoxLayout *layout = new QHBoxLayout(this);
-    layout->setContentsMargins(1, 1, 1, 1);
-    layout->setSpacing(0);
-    layout->addStretch();
-    layout->addWidget(m_capsIcon);
-    layout->addWidget(m_modeButton);
-    layout->addWidget(m_iconButton);
 
-    connect(m_edit, &QLineEdit::returnPressed, this, &IconEdit::clicked_cb);
-    connect(m_iconButton, &QPushButton::clicked, this, &IconEdit::clicked_cb);
-    connect(XEventMonitor::instance(), SIGNAL(keyRelease(const QString &)),
-            this, SLOT(onGlobalKeyRelease(const QString &)));
+    QtConcurrent::run([=](){
+        layout->setContentsMargins(1, 1, 1, 1);
+        layout->setSpacing(0);
+        layout->addStretch();
+        layout->addWidget(m_capsIcon);
+        layout->addWidget(m_modeButton);
+        layout->addWidget(m_iconButton);
 
-    setFocusProxy(m_edit);
-    setCapsState(checkCapsState());
+        m_edit->setAttribute(Qt::WA_InputMethodEnabled, false); //禁用输入法
+        m_edit->setContextMenuPolicy(Qt::NoContextMenu);    //禁用右键菜单
+        m_edit->installEventFilter(this);
+        //m_edit->setMaxLength(1000);
+
+        m_iconButton->setFocusPolicy(Qt::NoFocus);
+        m_iconButton->setCursor(QCursor(Qt::PointingHandCursor));
+        m_iconButton->installEventFilter(this);
+
+        m_modeButton->setCheckable(true);
+        m_modeButton->setFocusPolicy(Qt::NoFocus);
+        m_modeButton->setCursor(Qt::PointingHandCursor);
+        connect(m_modeButton, &QPushButton::clicked, this, [&](bool checked){
+            setType(checked ? QLineEdit::Normal : QLineEdit::Password);
+        });
+
+        connect(m_edit, &QLineEdit::returnPressed, this, &IconEdit::clicked_cb);
+        connect(m_iconButton, &QPushButton::clicked, this, &IconEdit::clicked_cb);
+        connect(XEventMonitor::instance(), SIGNAL(keyRelease(const QString &)),
+                this, SLOT(onGlobalKeyRelease(const QString &)));
+
+        setFocusProxy(m_edit);
+        setCapsState(checkCapsState());
+    });
 }
 
 bool IconEdit::eventFilter(QObject *obj, QEvent *event)
@@ -81,7 +89,10 @@ bool IconEdit::eventFilter(QObject *obj, QEvent *event)
     if(obj == m_edit){
         if(event->type() == QEvent::KeyPress){
             QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
-            if(keyEvent->key()==80 &&keyEvent->modifiers() ==(Qt::MetaModifier)){
+            if(keyEvent->matches(QKeySequence::Copy) || keyEvent->matches(QKeySequence::Cut) || keyEvent->matches(QKeySequence::Paste)){
+                event->ignore();
+                return true;
+            }else if((keyEvent->modifiers() & Qt::MetaModifier) || (keyEvent->modifiers() & Qt::AltModifier)){//当meta或者alt键被按下时，忽略按键事件
                 event->ignore();
                 return true;
             }
@@ -112,6 +123,10 @@ void IconEdit::setType(QLineEdit::EchoMode type)
 
 }
 
+void IconEdit::clearText()
+{
+    m_edit->setText("");
+}
 
 void IconEdit::resizeEvent(QResizeEvent *)
 {
@@ -172,7 +187,13 @@ void IconEdit::clear()
 
 void IconEdit::setPrompt(const QString &prompt)
 {
-    m_edit->setPlaceholderText(prompt);
+   // m_edit->setPlaceholderText(prompt);
+    if(prompt == "Password: " || prompt == "密码：")
+        m_edit->setPlaceholderText(tr("Password: "));
+    else if(prompt == "Username" || prompt == "用户名")
+        m_edit->setPlaceholderText(tr("Username"));
+    else
+        m_edit->setPlaceholderText(prompt);
 }
 
 const QString IconEdit::text()
@@ -198,6 +219,13 @@ void IconEdit::startWaiting()
     m_timer->start();
 }
 
+void IconEdit::refreshTranslate()
+{
+     if(m_edit->placeholderText() == "Password: " || m_edit->placeholderText() == "密码：")
+         m_edit->setPlaceholderText(tr("Password: "));
+     if(m_edit->placeholderText() == "Username" || m_edit->placeholderText() == "用户名")
+         m_edit->setPlaceholderText(tr("Username"));
+}
 
 void IconEdit::stopWaiting()
 {
