@@ -42,12 +42,15 @@ public:
     void startWaiting();
     void stopWaiting();
     void setCapsState(bool capsState);
+    void clearText();
 
 protected:
     void resizeEvent(QResizeEvent *) Q_DECL_OVERRIDE;
     bool eventFilter(QObject *obj, QEvent *event);
+
 private:
     void updatePixmap();
+    void refreshTranslate();
 
 signals:
     void clicked(const QString &);
